# -*- coding:utf-8 -*-

__title__ = 'sigal'
__author__ = 'Simon Conseil'
__version__ = '0.6.0'
__license__ = 'MIT'
__url__ = 'https://github.com/saimn/sigal'
__all__ = ['__title__', '__author__', '__version__', '__license__', '__url__']
